@@ -1,17 +1,19 @@
 #!/usr/bin/env python3
 # Copyright (c) 2020 oatsu
-"""
-Python3 module for HTS-full-label.
-Sinsy仕様のHTSフルコンテキストラベルを扱うモジュール
+"""Python3 module for HTS-full-label.
 
+Sinsy仕様のHTSフルコンテキストラベルを扱うモジュール
 forループが多いのでpypy3を使っても良いかも。(とくにdeepcopyが重い)
 """
 # import json
 import re
 from collections import UserList
-from copy import deepcopy
+from copy import copy, deepcopy
 from decimal import ROUND_HALF_UP, Decimal
 from itertools import chain
+from typing import Union
+
+from . import label as _label
 
 # from pprint import pprint
 
@@ -72,13 +74,12 @@ ABSPITCH_TO_NOTENUM_DICT = {
 
 
 def load(source):
-    """
-    HTSフルコンテキストラベル(Sinsy用)を読み取る
+    """HTSフルコンテキストラベル(Sinsy用)を読み取る
 
     source: path, lines
     """
-    song = HTSFullLabel()
-    return song.load(source)
+    full_label = HTSFullLabel()
+    return full_label.load(source)
 
 
 def notenum_to_abspitch(notenum) -> str:
@@ -106,7 +107,21 @@ class HTSFullLabel(UserList):
         super().__init__(init)
         self.song = Song()
 
-    def write(self, path, strict_sinsy_style: bool = True, label_type='full',
+    def as_mono(self) -> _label.Label:
+        """
+        モノラベルに変換する
+        """
+        # TODO: 内包表記かlambda式で高速化
+        mono_label = _label.Label()
+        for oneline in self:
+            mono_phoneme = _label.Phoneme()
+            mono_phoneme.start = copy(oneline.phoneme.start)
+            mono_phoneme.end = copy(oneline.phoneme.end)
+            mono_phoneme.symbol = copy(oneline.phoneme.identity)
+            mono_label.append(mono_phoneme)
+        return mono_label
+
+    def write(self, path, strict_sinsy_style: bool = True, as_mono: bool = False,
               mode='w', encoding='utf-8') -> str:
         """
         ファイル出力する
@@ -115,20 +130,13 @@ class HTSFullLabel(UserList):
             Trueのときは d, f における休符の長さ情報が削除されて 'xx' になる。
             Falseのときは d, f における休符の長さ情報が維持される。
         """
-        # モノラベルとして出力
-        if label_type == 'mono':
-            s = '\n'.join([f'{ol.start} {ol.end} {ol.phoneme.identity}' for ol in self])
-        # フルラベルとして出力
-        elif label_type == 'full':
-            # 休符周辺の仕様をSinsyに近づける。
-            new_label = adjust_pau_contexts(self, strict=strict_sinsy_style)
-            # 文字列にする
-            s = '\n'.join(list(map(str, new_label)))
-        # label_type が非対応の値だった時はエラー
-        else:
-            raise ValueError("Argument label_type must be 'full' or 'mono'.")
+        # 休符周辺の仕様をSinsyに近づける。
+        new_label = adjust_pau_contexts(self, strict=strict_sinsy_style)
+        # 文字列にする
+        s = '\n'.join(list(map(str, new_label)))
+
         # ファイル出力
-        with open(path, mode=mode, encoding=encoding) as f:
+        with open(path, mode=mode, encoding=encoding, newline='\n') as f:
             f.write(s)
         return s
 
@@ -160,7 +168,7 @@ class HTSFullLabel(UserList):
             with open(path, mode='r', encoding=encoding) as f:
                 lines = [line.rstrip('\r\n') for line in f.readlines()]
         except UnicodeDecodeError:
-            with open(path, mode='r', encoding='sjis') as f:
+            with open(path, mode='r', encoding='cp932') as f:
                 lines = [line.rstrip('\r\n') for line in f.readlines()]
         # 行ごとに分割したリストをもとに情報を登録する。
         self._load_from_lines(lines)
@@ -236,7 +244,7 @@ class HTSFullLabel(UserList):
 
     def fill_phonemes(self):
         """
-        phoneme をもとに、前後の音素に関する項を埋める。
+        Phoneme をもとに、前後の音素に関する項を埋める。
         """
         extended_self = [OneLine(), OneLine()] + self.data + [OneLine(), OneLine()]
         # ol is OneLine objec
@@ -358,6 +366,20 @@ class OneLine:
     def end(self, end_time: int):
         self.phoneme.end = end_time
 
+    @property
+    def duration(self) -> int:
+        """
+        発声時間の長さ (発声終了時刻-発声開始時刻)
+        """
+        return self.phoneme.end - self.phoneme.start
+
+    @property
+    def symbol(self) -> str:
+        """
+        音素記号
+        """
+        return self.phoneme.identity
+
     @property
     def p(self) -> list:
         """
@@ -567,19 +589,37 @@ class Song(UserList):
     def number_of_phrases(self, number: int):
         self.contexts[2] = number
 
-    def write(self, path, label_type='full', strict_sinsy_style: bool = True,
-              mode='w', encoding='utf-8') -> HTSFullLabel:
+    def as_mono(self) -> _label.Label:
+        """
+        モノラベルに変換する
+        """
+        mono_label = _label.Label()
+        for full_phoneme in self.all_phonemes:
+            mono_phoneme = _label.Phoneme()
+            mono_phoneme.start = copy(full_phoneme.start)
+            mono_phoneme.end = copy(full_phoneme.end)
+            mono_phoneme.symbol = copy(full_phoneme.identity)
+            mono_label.append(mono_phoneme)
+        return mono_label
+
+    def write(self, path, strict_sinsy_style: bool = True, as_mono: bool = False,
+              mode='w', encoding='utf-8') -> Union[HTSFullLabel, _label.Label]:
         """
         ファイル出力する。
         HTSFullLabelからではなく、USTなどからSongが直接生成されている場合に対応する。
         """
         full_label = HTSFullLabel()
         full_label.song = self
+
         full_label.fill_contexts_from_songobj()
+        if as_mono is True:
+            mono_label = full_label.as_mono()
+            mono_label.write(path, mode=mode, encoding=encoding)
+            return mono_label
+
         full_label.write(
-            path, label_type=label_type, strict_sinsy_style=strict_sinsy_style,
-            mode=mode, encoding=encoding
-        )
+            path, strict_sinsy_style=strict_sinsy_style,
+            mode=mode, encoding=encoding)
         return full_label
 
     def reset_time(self):
@@ -600,20 +640,30 @@ class Song(UserList):
                 phoneme.end = Decimal(t_end).quantize(Decimal('0'), rounding=ROUND_HALF_UP)
             t_start = t_end
 
-    def autofill(self):
+    def autofill(self, hts_conf: Union[None, dict] = None):
         """
         自動補完可能なものをすべて自動補完する。
         """
         # この順でやらないと、p1が未設定の状態で Note.is_rest() をやってしまう
-        self._fill_phoneme_contexts()
+        self._fill_phoneme_contexts(hts_conf)
         self._fill_syllable_contexts()
         self._fill_note_contexts()
         self._fill_song_contexts()
 
-    def _fill_phoneme_contexts(self, vowels=VOWELS, pauses=PAUSES, silences=SILENCES,breaks=BREAKS):
+    def _fill_phoneme_contexts(self, hts_conf: Union[None, dict]):
         """
         p1, p12, p13, p14, p15 を補完する。
         """
+        # p12, p13, p14, p15を埋める
+        if hts_conf is None:
+            self._fill_p1(vowels=VOWELS, pauses=PAUSES, silences=SILENCES, breaks=BREAKS)
+        else:
+            self._fill_p1(vowels=hts_conf['VOWELS'], pauses=hts_conf['PAUSES'],
+                          silences=hts_conf['SILENCES'], breaks=hts_conf['BREAKS'])
+        self._fill_p12_p13()
+        self._fill_p14_p15()
+
+    def _fill_p1(self, vowels, pauses, silences, breaks):
         # p1 を埋める
         for phoneme in self.all_phonemes:
             phoneme_identity = phoneme.identity
@@ -629,9 +679,6 @@ class Song(UserList):
                 phoneme.language_independent_identity = 'b'
             else:
                 phoneme.language_independent_identity = 'c'
-        # p12, p13, p14, p15を埋める
-        self._fill_p12_p13()
-        self._fill_p14_p15()
 
     def _fill_p12_p13(self):
         """
@@ -695,8 +742,7 @@ class Song(UserList):
                 syllable.position_backward = len_note - i
 
     def _fill_note_contexts(self):
-        """
-        e を補完する。
+        """e を補完する。
 
         必要なデータ:
             - e1: 絶対音高
@@ -791,8 +837,8 @@ class Song(UserList):
                 counter_100ns += note.length_100ns
             # 休符でもなくて最初のノートでもないとき
             else:
-                note.position_100ms = \
-                    Decimal(counter_100ns / 1000000).quantize(Decimal('0'), rounding=ROUND_HALF_UP)
+                note.position_100ms = Decimal(
+                    counter_100ns / 1000000).quantize(Decimal('0'), rounding=ROUND_HALF_UP)
                 note.position_100ns = counter_100ns
                 counter_100ns += note.length_100ns
 
@@ -807,19 +853,19 @@ class Song(UserList):
             # フレーズ中で最後のノートのときは時間をリセットして、累積時間を増やす。
             elif note.position_backward == 1:
                 counter_100ns = note.length_100ns
-                note.position_100ms_backward = \
-                    Decimal(counter_100ns / 1000000).quantize(Decimal('0'), rounding=ROUND_HALF_UP)
+                note.position_100ms_backward = Decimal(
+                    counter_100ns / 1000000).quantize(Decimal('0'), rounding=ROUND_HALF_UP)
                 note.position_100ns_backward = counter_100ns
             # 休符でもなくて最後のノートでもないとき
             else:
                 counter_100ns += note.length_100ns
-                note.position_100ms_backward = \
-                    Decimal(counter_100ns / 1000000).quantize(Decimal('0'), rounding=ROUND_HALF_UP)
+                note.position_100ms_backward = Decimal(
+                    counter_100ns / 1000000).quantize(Decimal('0'), rounding=ROUND_HALF_UP)
                 note.position_100ns_backward = counter_100ns
 
     def _fill_e22_e23(self):
-        """
-        「フレーズ内での位置(96分音符単位) (e22, e23)」の項を埋める。
+        """「フレーズ内での位置(96分音符単位) (e22, e23)」の項を埋める。
+
         e18とe19のデータがある前提で実行する。
         """
         counter = 0
@@ -907,7 +953,6 @@ class Song(UserList):
         #         note.contexts[24] = Decimal(100 * counter_100ns / phrase_length_100ns
         #                                     ).quantize(Decimal('0'), rounding=ROUND_HALF_UP)
 
-
     def _fill_e57_e58(self):
         """
         「前後のノートとの音高差 (e57, e58)」の項を埋める。
@@ -950,9 +995,9 @@ class Song(UserList):
         """
         Songオブジェクトのコンテキストを自動補完する。
         """
-        self.fill_j3()
+        self._fill_j3()
 
-    def fill_j3(self):
+    def _fill_j3(self):
         """
         Songオブジェクト内のフレーズ数(j3) の項を埋める。
         """
@@ -973,8 +1018,8 @@ class Song(UserList):
 
 
 class Phrase(UserList):
-    """
-    フレーズを扱うクラス
+    """フレーズを扱うクラス
+
     昨日のフレーズ G (g1~g2)
     今日のフレーズ H (h1~h2)
     明日のフレーズ I (i1~i2)
@@ -1307,6 +1352,13 @@ class Phoneme:
     def __str__(self):
         return f'{self.start} {self.end} {self.identity}'
 
+    @property
+    def duration(self) -> int:
+        """
+        discription
+        """
+        return self.end - self.start
+
     def is_vowel(self):
         """
         母音かどうか
@@ -1326,7 +1378,8 @@ class Phoneme:
         return self.language_independent_identity in ('s', 'p')
 
     def is_pau(self):
-        """
+        """休符かどうかを判定
+
         後方互換のために残した。
         'is_pau' is now 'is_rest'
         """
